import os

from spacy.lang.en import English
from spacy.lang.xx import MultiLanguage

from mudes.algo.mudes_model import MUDESModel
import logging
from google_drive_downloader import GoogleDriveDownloader as gdd

from mudes.algo.predict import predict_spans
from mudes.algo.preprocess import contiguous_ranges


class PredictedToken:
    def __init__(self, text, is_toxic):
        self.text = text
        self.is_toxic = is_toxic


class MUDESApp:
    def __init__(self, model_name_or_path, model_type=None,  use_cuda=True,  cuda_device=-1):

        self.model_name_or_path = model_name_or_path
        self.model_type = model_type
        self.use_cuda = use_cuda
        self.cuda_device = cuda_device

        MODEL_CONFIG = {
            "en-base": ("bert", "mudes/en-base"),
            "en-large": ("roberta", "mudes/en-large"),
            "multilingual-base": ("xlmroberta", "mudes/multilingual-base"),
            "multilingual-large": ("xlmroberta", "mudes/multilingual-large")
        }

        if model_name_or_path in MODEL_CONFIG:
            self.trained_model_type, self.model_id = MODEL_CONFIG[model_name_or_path]

            self.model = MUDESModel(self.trained_model_type, self.model_id, labels=["NOT_TOXIC", "TOXIC"], use_cuda=self.use_cuda,
                                        cuda_device=self.cuda_device)

        else:
            self.model = MUDESModel(model_type, self.model_name_or_path, labels=["NOT_TOXIC", "TOXIC"], use_cuda=self.use_cuda,
                                        cuda_device=self.cuda_device)

    @staticmethod
    def _download(drive_id, model_name):
        gdd.download_file_from_google_drive(file_id=drive_id,
                                            dest_path= os.path.join(".mudes", model_name, "model.zip"),
                                            unzip=True)

    def predict_toxic_spans(self, text: str, spans: bool = False, language: str = "en"):
        toxic_spans = predict_spans(self.model, text, language)
        if spans:
            return contiguous_ranges(toxic_spans)
        else:
            return toxic_spans

    def predict_tokens(self, text: str, language: str = "en"):
        toxic_spans = contiguous_ranges(predict_spans(self.model, text))

        if language == "en":
            nlp = English()

        else:
            nlp = MultiLanguage()

        tokenizer = nlp.Defaults.create_tokenizer(nlp)
        tokens = tokenizer(text)
        output_tokens = []
        for token in tokens:
            is_toxic = False
            for toxic_span in toxic_spans:
                if toxic_span[0] <= token.idx <= toxic_span[1]:
                    is_toxic = True
                    break

            predicted_token = PredictedToken(token.text, is_toxic)
            output_tokens.append(predicted_token)

        return output_tokens





import os
import shutil
import statistics

from sklearn.model_selection import train_test_split

from examples.english.transformer_configs import transformer_config, MODEL_TYPE, MODEL_NAME, LANGUAGE_FINETUNE, \
    language_modeling_args, TEMP_DIRECTORY
from mudes.algo.evaluation import f1
from mudes.algo.mudes_model import MUDESModel
from mudes.algo.language_modeling import LanguageModelingModel
from mudes.algo.predict import predict_spans
from mudes.algo.preprocess import read_datafile, format_data, format_lm, split_data, read_test_datafile
import torch
import pandas as pd
import numpy as np

if not os.path.exists(TEMP_DIRECTORY):
    os.makedirs(TEMP_DIRECTORY)

train = read_datafile('examples/english/data/tsd_train.csv')
dev = read_datafile('examples//english/data/tsd_trial.csv')
test = read_test_datafile('examples//english/data/tsd_test.csv')


if LANGUAGE_FINETUNE:
    train_list = format_lm(train)
    dev_list = format_lm(dev)
    test_list = format_lm(test, test=True)

    complete_list = train_list + dev_list + test_list

    lm_train = complete_list[0: int(len(complete_list)*0.8)]
    lm_test = complete_list[-int(len(complete_list)*0.2):]

    with open(os.path.join(TEMP_DIRECTORY, "lm_train.txt"), 'w') as f:
        for item in lm_train:
            f.write("%s\n" % item)

    with open(os.path.join(TEMP_DIRECTORY, "lm_test.txt"), 'w') as f:
        for item in lm_test:
            f.write("%s\n" % item)

    model = LanguageModelingModel("auto", MODEL_NAME, args=language_modeling_args, use_cuda=torch.cuda.is_available())
    model.train_model(os.path.join(TEMP_DIRECTORY, "lm_train.txt"), eval_file=os.path.join(TEMP_DIRECTORY, "lm_test.txt"))
    MODEL_NAME = language_modeling_args["best_model_dir"]


# model = HateSpansModel(MODEL_TYPE, MODEL_NAME, labels=tags, args=transformer_config)
dev_preds = []
test_preds = []
fold_preds = []
for i in range(transformer_config["n_fold"]):
    if os.path.exists(transformer_config['output_dir']) and os.path.isdir(transformer_config['output_dir']):
        shutil.rmtree(transformer_config['output_dir'])
    print("Started Fold {}".format(i))

    if transformer_config["evaluate_during_training"]:
        train_list, val_list = split_data(train, seed=int(transformer_config["manual_seed"])*(i+1))
        train_df = format_data(train_list)
        val_df = format_data(train_list)
        tags = train_df['labels'].unique().tolist()
        model = MUDESModel(MODEL_TYPE, MODEL_NAME, labels=tags, args=transformer_config)
        model.train_model(train_df, eval_df=val_df)
        model = MUDESModel(MODEL_TYPE, transformer_config["best_model_dir"], labels=tags, args=transformer_config)

    else:
        train_df = format_data(train)
        tags = train_df['labels'].unique().tolist()
        model = MUDESModel(MODEL_TYPE, MODEL_NAME, labels=tags, args=transformer_config)

    scores = []
    dev_predictions_list = []
    for n, (spans, text) in enumerate(dev):
        predictions = predict_spans(model, text)
        score = f1(predictions, spans)
        scores.append(score)
        dev_predictions_list.append(predictions)

    fold_preds.append(statistics.mean(scores))
    dev_preds.append(dev_predictions_list)
    print('avg F1 %g' % statistics.mean(scores))

    test_predictions_list = []
    for n, text in enumerate(test):
        predictions = predict_spans(model, text)
        test_predictions_list.append(predictions)

    test_preds.append(test_predictions_list)

print('avg F1 scores in each fold', fold_preds)
scores = []
for n, (spans, text) in enumerate(dev):
    majority_span = []
    # fold_predictions = dev_preds[n]
    fold_predictions = []
    for prediction_list in dev_preds:
        fold_predictions.append(prediction_list[n])
    for index in range(0, len(text)):
        count = 0
        for fold_prediction in fold_predictions:
            if index in fold_prediction:
                count += 1
        if count/transformer_config["n_fold"] >= 0.5:
            majority_span.append(index)
    score = f1(majority_span, spans)
    scores.append(score)

print('avg F1 %g' % statistics.mean(scores))

final_predictions = []
for n, text in enumerate(test):
    majority_span = []
    fold_predictions = []
    for prediction_list in test_preds:
        fold_predictions.append(prediction_list[n])
    for index in range(0, len(text)):
        count = 0
        for fold_prediction in fold_predictions:
            if index in fold_prediction:
                count += 1
        if count/transformer_config["n_fold"] >= 0.5:
            majority_span.append(index)
    final_predictions.append(majority_span)


with open(os.path.join(TEMP_DIRECTORY, "spans-pred.txt"), "w") as out:
    for idx, prediction in enumerate(final_predictions):
        out.write(f"{str(idx)}\t{str(prediction)}\n")













import pandas as pd

from mudes.algo.evaluation import binary_macro_f1, binary_weighted_f1
from mudes.app.mudes_app import MUDESApp


test = pd.read_csv('data/test_a_tweets.tsv', sep="\t")
test = test.rename(columns={'tweet': 'text'})
test = test[['text']]

test_sentences = test['text'].tolist()

app = MUDESApp("small", use_cuda=False)

toxic_spans_list = []
predictions = []
for test_sentence in test_sentences:
    toxic_spans = app.predict_toxic_spans(test_sentence, spans=True)
    toxic_spans_list.append(toxic_spans)

    if len(toxic_spans) > 0:
        predictions.append("OFF")

    else:
        predictions.append("NOT")

labels = pd.read_csv('data/test_a_labels.csv', sep=",", names=['id', 'label'], header=None)
print("Macro F1 ", binary_macro_f1(labels['label'].tolist(), predictions) )
print("Weighted F1 ", binary_weighted_f1(labels['label'].tolist(), predictions) )





from setuptools import find_packages, setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="mudes",
    version="0.2.2-beta",
    author="Tharindu Ranasinghe",
    author_email="rhtdranasinghe@gmail.com",
    description="Toxic Spans Prediction",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/TharinduDR/MUDES",
    packages=find_packages(exclude=("examples",)),
    classifiers=[
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
    python_requires=">=3.6",
    install_requires=[
        "numpy",
        "requests",
        "tqdm>=4.47.0",
        "regex",
        "transformers>=4.2.0",
        "scipy",
        "scikit-learn",
        "tensorboardx",
        "pandas",
        "tokenizers",
        "matplotlib",
        "wandb",
        "sentencepiece",
        "onnxruntime",
        "seqeval",
    ],
)



from mudes.app.mudes_app import MUDESApp

# app = HateSpansApp("multilingual-large", use_cuda=False)
# print(app.predict_hate_spans("ගෝඨාභය පොන්නයා විවාදෙට එන්නේ නැත්තේ ඇයි දන්නවා ද?", spans=True, language="xx"))
#
# tokens = app.predict_tokens("ගෝඨාභය පොන්නයා විවාදෙට එන්නේ නැත්තේ ඇයි දන්නවා ද?", language="xx")
# for token in tokens:
#     print(token.text, token.is_toxic)

app = MUDESApp("en-base", use_cuda=False)
print(app.predict_toxic_spans("Fuck this motherfucker", spans=True, language="en"))

tokens = app.predict_tokens("Fuck this motherfucker", language="en")
for token in tokens:
    print(token.text, token.is_toxic)